import { useReducer } from 'react';
export function useReducerWithMiddleware(reducer, initialState) {
    return function (middlewares) {
        var _a = useReducer(reducer, initialState), store = _a[0], dispatch = _a[1];
        var api = {
            getState: function () { return store; },
            dispatch: function (action) { return dispatch(action); },
        };
        var chain = middlewares.map(function (middleware) { return middleware(api); });
        return {
            store: store,
            dispatch: compose(chain)(dispatch),
        };
    };
}
var compose = function (fns) {
    if (fns.length === 0) {
        return function (_) { return function (action) { return action; }; };
    }
    if (fns.length === 1) {
        return fns[0];
    }
    var last = fns[fns.length - 1];
    var rest = fns.slice(0, fns.length - 1);
    return function (dispatch) {
        return rest.reduceRight(function (composed, f) { return f(composed); }, last(dispatch));
    };
};
